package controllers

import (
	"encoding/json"
	"hongze/hongze_ETA_mobile_api/models"
	"hongze/hongze_ETA_mobile_api/models/system"
	"hongze/hongze_ETA_mobile_api/utils"
	"time"
)

// 系统用户
type SysAdminController struct {
	BaseAuthController
}

// SetConfig
// @Title 用户信息配置接口
// @Description 用户信息配置接口
// @Success 200 {object} system.SetAdminConfigReq
// @router /sysuser/config/set [post]
func (this *SysAdminController) SetConfig() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req system.SetAdminConfigReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	item, err := system.GetConfigDetailByCode(this.SysUser.AdminId, req.ConfigCode)
	if err != nil {
		if err.Error() != utils.ErrNoRow() {
			br.Msg = "设置失败"
			br.ErrMsg = "设置失败，Err:" + err.Error()
			return
		}

		// 获取默认配置
		defaultConfig, err := system.GetDefaultConfigDetailByCode(req.ConfigCode)
		if err != nil {
			br.Msg = "设置失败"
			br.ErrMsg = "获取默认配置失败，Err:" + err.Error()
			return
		}
		adminConfigItem := &system.AdminConfig{
			//ConfigId:    0,
			AdminId:     this.SysUser.AdminId,
			ConfigCode:  req.ConfigCode,
			ConfigValue: req.ConfigValue,
			Remark:      defaultConfig.Remark,
			CreateTime:  time.Now(),
		}
		err = adminConfigItem.Create()

		if err != nil {
			br.Msg = "设置失败"
			br.ErrMsg = "设置失败，Err:" + err.Error()
			return
		}
	} else {
		// 存在配置，那么就修改
		item.ConfigValue = req.ConfigValue
		err = item.Update([]string{"ConfigValue"})

		if err != nil {
			br.Msg = "设置失败"
			br.ErrMsg = "设置失败，Err:" + err.Error()
			return
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "设置成功"
}
